# Maps for TFM

library(urbanR)
library(RNetCDF)
library(sp)
library(sf)
#library(mapview)
library(ncdf4)
library(rgeos) 
library(ggplot2)
library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr 
library(dplyr) 
library(broom)
library(magrittr)
library(ggpubr)
library(raster)
library(rgdal)
#library(tmap)
library(gstat)
library(sp)
library(abind)
library(openair)
library(tidyr)
#library(RColorBrewer)
#library(Metrics)
library(gbm)
library(lubridate)


library(matrixStats)
#-------------------------------------------------------------------------------
pollutants       <- c("pm2p5")
years            <- c(2019) 


general_path <- "/esarchive/scratch/abarrant/results_regional_"
save_sp      <- "/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/stats_results/"


compute_in_UTM   <- "TRUE"
crs_latlon       <- "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0"
crs_utm          <- "+proj=utm +zone=31 +ellps=intl +units=m +no_defs"



my_box            <- rgeos::bbox2SP(43.02355,40.24870,3.497162,-0.001190186, proj4string = CRS(crs_latlon)) # lat màx, lat min, lon max, lon min
# Generate an uniform mesh
receptor_grid                     <- sf::st_sf(geom=sf::st_make_grid(my_box, n=c(300,300), what = "centers"), crs=crs_latlon)
sp_receptor_grid                  <- sf::as_Spatial(receptor_grid)
sp::proj4string(sp_receptor_grid) <- crs_latlon

if (compute_in_UTM) {
  sp_receptor_grid <- spTransform(sp_receptor_grid, sp::CRS(crs_utm))
}

#rang <- c(0,10,20,30,40,80,120,160,200,300,400)
rang <-c(0, 10, 20,30,35,40,45,50,60,70,80,90)

for( p in 1: length(pollutants)){
  #p<- 1
  pollutant     <- pollutants[p]
  save_final_sp <- paste0(save_sp,pollutant,"_matrix")
  
  
  for(y in 1:length(years)){
    #y <- 1
    year            <- years[y]
    pollutant_path  <- paste0(general_path,year,"/results_gene_",pollutant, "/output")
    #path_model      <- paste0(pollutant_path,"/UK_gene_",pollutant,"_df.rds")            #model corregit amb uk sencer?
    path_model      <- paste0(pollutant_path,"/model_gene_",pollutant,".rds")  
    path_coords     <- paste0(pollutant_path,"/coords_model_gene_",pollutant,".rds")
    
    coords_model    <- readRDS(file = path_coords)
    
    lon_model       <- coords_model[, "lon_model"]
    lat_model       <- coords_model[, "lat_model"]
    model_corrected <- readRDS(file=path_model)
    
    
    length_data     <- length(model_corrected)
    n_points_model  <- length(model_corrected[[1]])
    
    
    if(pollutant=="no2" | pollutant=="o3"){
      n_max       <- 19
      length_days <- length_data/24
    } else if (pollutant=="pm10" | pollutant=="pm2p5"){
      n_max       <- 36
      length_days <- length_data
      }
      
      pollutant_matrix <- matrix(data=NA, nrow=n_points_model,ncol = length_data )
      
      
      for(h in 1:length_data) {
        pollutant_matrix[,h] <-  model_corrected[[h]]
      }
      
      #Annual mean
      annual_mean                      <- rowMeans(pollutant_matrix, na.rm = TRUE)
      df_annual_mean                   <- data.frame(pollutant=annual_mean, lon=lon_model, lat=lat_model)
      coordinates(df_annual_mean)      <- ~lon+lat
      sp::proj4string(df_annual_mean)  <- crs_latlon
      
      if (compute_in_UTM) {
        df_annual_mean <- spTransform(df_annual_mean, sp::CRS(crs_utm))
      }
      
      set.seed(1) #we are picking random points in the next line, set.seed permits us to have same points in each loop
      pick_random_points <- sample(1:nrow(df_annual_mean),1000)
      df_annual_mean     <- na.omit(df_annual_mean)
      #pollutant.fit_ok   <- automap::autofitVariogram(formula=pollutant~1, input_data=df_annual_mean[pick_random_points,],model="Ste")
      pollutant.fit_ok   <- automap::autofitVariogram(formula=pollutant~1, input_data=df_annual_mean,model="Ste")
      ok_gstat           <- gstat(formula=pollutant~1, data = df_annual_mean, model = pollutant.fit_ok$var_model, nmax=300)
      OK                 <- predict(object=ok_gstat, newdata=sp_receptor_grid)
      
      pixels      <- sp::SpatialPixelsDataFrame(OK,tolerance = 0.99, as.data.frame(OK[,"var1.pred"]))
      #pixels      <- sp::SpatialPixelsDataFrame(OK,tolerance = 0.99, as.data.frame(OK@data$var1.pred))
      mean_raster <- raster::raster(pixels[,'var1.pred'])
      
      writeOGR(mean_raster, dsn = paste0(save_final_sp,"/annual_mean_",pollutant, "_",year,".shp"), layer = "capa", driver = "ESRI Shapefile")
      datos_leidos <- readOGR(dsn = paste0(save_final_sp,"/annual_mean_",pollutant, "_",year,".shp"), layer = paste0("annual_mean_pm10_2020"))
      
      # mapview::mapview(OK$var1.pred, at=rang)
      # mapview::mapview(datos_leidos, at=rang)
      mapview::mapview(mean_raster, at=rang)
      # mapview::mapview(pixels, zcol="var1.pred", at=rang)
      
      #36 valor max / 19 è
      if(pollutant=="pm10" | pollutant=="pm2p5" | pollutant=="no2"){
        for(i in 1:n_points_model){
          pollutant_sort[i,] <- sort(pollutant_matrix[i,],na.last=TRUE,decreasing=TRUE)
          pollutant_sort[i,] <- na.omit(pollutant_sort[i,])
          pollutant_max[i]   <- pollutant_sort[i,n_max]
        }
        
        df_pollutant_max                   <- data.frame(data=pollutant_max,lon=lon_model,lat=lat_model)
        coordinates(df_pollutant_max)      <- ~lon+lat
        sp::proj4string(df_pollutant_max)  <- crs_latlon
        
        if (compute_in_UTM) {
          df_pollutant_max <- spTransform(df_pollutant_max, sp::CRS(crs_utm))
        }
      
        
        set.seed(1) #we are picking random points in the next line, set.seed permits us to have same points in each loop
        pick_random_points <- sample(1:nrow(df_pollutant_max),1000)
        
        pollutant.fit_ok   <- automap::autofitVariogram(formula=pollutant~1, input_data=df_pollutant_max[pick_random_points,],model="Ste")
        ok_gstat           <- gstat(formula=pollutant~1, data = df_pollutant_max, model = pollutant.fit_ok$var_model, nmax=10)
        OK                 <- predict(object=ok_gstat, newdata=sp_receptor_grid)
        
        pixels_max      <- sp::SpatialPixelsDataFrame(OK,tolerance = 0.99, as.data.frame(OK[,"var1.pred"]))
        mean_raster_max <- raster::raster(pixels_max[,'var1.pred'])
        
        writeOGR(mean_raster_max, dsn = paste0(save_final_sp,"/annual_",n_max,"_",pollutant, "_",year,".shp"), layer = "capa", driver = "ESRI Shapefile")
        #datos_leidos <- readOGR(dsn = paste0(save_final_sp,"/annual_mean_",pollutant, "_",year,".shp"), layer = paste0("annual_mean_pm10_2020"))
      
      } else if(pollutant=="o3"){
        # media octohoraria
        # OCTOHORARI
        media_octohoraria_mean  <- matrix(NA, nrow = n_points_model, ncol = 17) #media octohoraria day canviar
        media_octohoraria_max   <- matrix(NA, nrow = n_points_model, ncol = length_days)
        annual_26_max           <- matrix(NA,nrow=n_points_model, ncol=1)
        
        for (i in 1:length_days) {
          inicio                   <- (i - 1) * 24 + 1
          fin                      <- i * 24
          bloques_24h              <- pollutant_matrix[,inicio:fin]
          #la legislacio estableix un nombre màx de NA per poder fer aquesta mitjana (treball futur)
          for(o in 1:17){
            start    <- o
            end      <- o+7
            bloque_8h                  <- bloques_24h[,start:end]
            mean_8h                    <- apply(bloque_8h, 1, mean, na.rm = TRUE) #mirar el 1 d'on  ve
            media_octohoraria_mean[,o] <- mean_8h
          }
          media_octohoraria_max [,i]<- rowMaxs(media_octohoraria_mean, na.rm = TRUE)
        }
        annual_26_max <- sort(media_octohoraria_max, decreasing = TRUE)[26] #revisar si funciona aquesta linea
        
        df_ozone_8                   <- data.frame(data=annual_26_max,lon=lon_model,lat=lat_model)
        coordinates(df_ozone_8)      <- ~lon+lat
        sp::proj4string(df_ozone_8)  <- crs_latlon
        
        if (compute_in_UTM) {
          df_ozone_8 <- spTransform(df_ozone_8, sp::CRS(crs_utm))
        }
        
        
        set.seed(1) #we are picking random points in the next line, set.seed permits us to have same points in each loop
        pick_random_points <- sample(1:nrow(df_ozone_8),1000)
        
        pollutant.fit_ok_o3   <- automap::autofitVariogram(formula=pollutant~1, input_data=df_ozone_8[pick_random_points,],model="Ste")
        ok_gstat_o3           <- gstat(formula=pollutant~1, data = df_ozone_8, model = pollutant.fit_ok_o3$var_model, nmax=10)
        OK_o3                 <- predict(object=ok_gstat_o3, newdata=sp_receptor_grid)
        
        pixels_ozone      <- sp::SpatialPixelsDataFrame(OK_o3,tolerance = 0.99, as.data.frame(OK[,"var1.pred"]))
        mean_raster_ozone <- raster::raster(pixels_ozone[,'var1.pred'])
        
        writeOGR(mean_raster_ozone, dsn = paste0(save_final_sp,"/annual_","26_octohorari_",pollutant, "_",year,".shp"), layer = "capa", driver = "ESRI Shapefile")
        #datos_leidos <- readOGR(dsn = paste0(save_final_sp,"/annual_mean_",pollutant, "_",year,".shp"), layer = paste0("annual_mean_pm10_2020"))
      }
        
  }
  
}





#-------------------------------------------------------------------------------
# model_corrected <- UK_total_df
# length_data     <- length(model_corrected)
# n_points_model  <- length(model_corrected[[1]])
# 
# if(pollutant=="no2"){
#   n_max <- 19
#   length_days <- length_data/24
# } else if (pollutant=="pm10" | pollutant=="pm2p5"){
#   n_max <- 36
#   length_days <- length_data
# }
# 
# pollutant_matrix <- matrix(data=NA, nrow=n_points_model,ncol = length_data )
# 
# 
# for(h in 1:length_data) {
#   pollutant_matrix[,h] <-  model_corrected[[h]]
# }
# 
# if(pollutant=="o3"){
#   # ANNUAL MEAN
#   annual_mean                      <- rowMeans(pollutant_matrix, na.rm = TRUE)
#   df_annual_mean                   <- data.frame(data=annual_mean, lon=lon_model, lat=lat_model)
#   coordinates(df_annual_mean)      <- ~lon+lat
#   sp::proj4string(df_annual_mean)  <- crs_latlon
#   
#   # OCTOHORARI
#   media_octohoraria_mean  <- matrix(NA, nrow = n_points_model, ncol = 17) #media octohoraria day canviar
#   media_octohoraria_max   <- matrix(NA, nrow = n_points_model, ncol = length_days)
#   annual_26_max           <- matrix(NA,nrow=n_points_model, ncol=1)
#   
#   for (i in 1:length_days) {
#     inicio                   <- (i - 1) * 24 + 1
#     fin                      <- i * 24
#     bloques_24h              <- pollutant_matrix[,inicio:fin]
#     #la legislacio estableix un nombre màx de NA per poder fer aquesta mitjana (treball futur)
#     for(o in 1:17){
#       start    <- o
#       end      <- o+7
#       bloque_8h                  <- bloques_24h[,start:end]
#       mean_8h                    <- apply(bloque_8h, 1, mean, na.rm = TRUE) #mirar el 1 d'on  ve
#       media_octohoraria_mean[,o] <- mean_8h
#     }
#     media_octohoraria_max [,i]<- rowMaxs(media_octohoraria_mean, na.rm = TRUE)
#   }
#   annual_26_max <- sort(media_octohoraria_max, decreasing = TRUE)[26] #revisar si funciona aquesta linea
# }
# 
# 
# if (pollutant!="o3"){
#   # ANNUAL MEAN
#   annual_mean                      <- rowMeans(pollutant_matrix, na.rm = TRUE)
#   df_annual_mean                   <- data.frame(data=annual_mean, lon=lon_model, lat=lat_model)
#   coordinates(df_annual_mean)      <- ~lon+lat
#   sp::proj4string(df_annual_mean)  <- crs_latlon
#   
#   if (compute_in_UTM) {
#     df_annual_mean <- spTransform(df_annual_mean, sp::CRS(crs_utm))
#   }
#   
#   mean_uniform          <- interp(x=df_annual_mean,z="data",nx=400,ny=400) #int the means of the model to an uniform "malla" of 1000x1000
#   mapview(mean_uniform, legend = TRUE, at = c(0,10,20,30,35,40,45,50,60,70,80))
#   
#   mapview::mapview(mean_uniform)
#   
#   # The n_max max value
#   pollutant_sort              <- pollutant_matrix
#   pollutant_max               <- base::rep(NA,n_points_model)
#   
#   if(pollutant=="pm10" | pollutant=="pm2p5"){
#     for(i in 1:n_points_model){
#       pollutant_sort[i,]=sort(pollutant_matrix[i,],na.last=TRUE,decreasing=TRUE)
#       pollutant_max[i]=pollutant_sort[i,36]
#     }
#     
#     df_pollutant_max                   <- data.frame(data=pollutant_max,lon=lon_model,lat=lat_model)
#     coordinates(df_pollutant_max)      <- ~lon+lat
#     sp::proj4string(df_pollutant_max)  <- crs_latlon
#     
#     if (compute_in_UTM) {
#       df_pollutant_max <- spTransform(df_pollutant_max, sp::CRS(crs_utm))
#     }
#   } else if(pollutant=="no2"){ #falç fer = pm10
#     
#     #Replantejament 19 è max
#     #pollutant_day         <- matrix(NA, nrow=n_points_model, ncol=24)
#     pollutant_annual_max  <- matrix(NA, nrow=n_points_model, ncol=length_days)
#     pollutant_19_max      <- matrix(NA, nrow=n_points_model, ncol=1)
#     
#     
#     for (i in 1:length_days) {
#       inicio                   <- (i - 1) * 24 + 1
#       fin                      <- i * 24
#       bloques_24h              <- pollutant_matrix[,inicio:fin]
#       maximos                  <- apply(bloques_24h, 1, max, na.rm = TRUE)
#       pollutant_annual_max[,i] <- maximos
#     }
#     
#     for(n in 1:n_points_model){
#       pollutant_19_max[n, ] <- sort(pollutant_annual_max[i, ], decreasing = TRUE)[19]
#       
#     }
#     df_pollutant_19_max <- data.frame(data=pollutant_19_max, lon=lon_model, lat=lat_model)
#     coordinates(df_pollutant_19_max) <- ~lon+lat
#     sp::proj4string(df_pollutant_19_max)  <- crs_latlon
#     
#     if (compute_in_UTM) {
#       df_pollutant_19_max <- spTransform(df_pollutant_19_max, sp::CRS(crs_utm))
#     }
#   }
#   
#   
# }





# # Màx.
# 
# pollutant_max               <- base::rep(NA,n_points_model)
# 
# #màxim
# for(i in 1:n_points_model){
#   pollutant_max[i]=max(pollutant_matrix[i,],na.rm=TRUE) }
# 
# df_pollutant_max                   <- data.frame(data=pollutant_max,lon=lon_model, lat=lat_model)
# coordinates(df_pollutant_max)      <- ~lon+lat
# sp::proj4string(df_pollutant_max)  <- crs_latlon
# 
# if (compute_in_UTM) {
#   df_pollutant_max <- spTransform(df_pollutant_max, sp::CRS(crs_utm))
# }
# 
# # Transponer los datos
# data <- UK_total_df
# 
# # Obtener dimensiones de los datos transpuestos
# num_points <- length(data[[1]])
# num_hours <- length(data)
# 
# # Definir el número de días y calcular el número de bloques de 24 horas
# num_days <- num_hours %/% 24
# num_blocks <- num_days * 24
# 
# # Crear una matriz para almacenar los máximos diarios
# matriz_maximos <- matrix(0, nrow = num_points, ncol = num_days)
# 
# # Calcular los máximos diarios
# for (i in 1:num_days) {
#   inicio              <- (i - 1) * 24 + 1
#   fin                 <- i * 24
#   bloques_24h         <- data[inicio:fin]
#   maximos             <- apply(bloques_24h, 1, max)
#   matriz_maximos[, i] <- maximos
# }


