# Needs packages:
#library(urbanR)
#library(gbm)
library(lubridate)
library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr 
library(dplyr) #per treballar amb cbinds i esas cosas
#library(caret)
library(zoo)
library(scatterplot3d)
library(plotly)

#borrar
#library(RNetCDF)
#library(sp)
library(sf)
library(mapview)
#library(ncdf4)
#library(rgeos) 
library(ggplot2)
#library(broom)
#library(magrittr)
#library(ggpubr)
#library(raster)
#library(rgdal)
#library(tmap)
library(gstat)
#library(abind)
#library(rbind)
library(openair)
#library(tidyr)

############################################################
#                                                          #
#                    Initial data                          #
#                                                          #
############################################################
pollutant       <- "pm2p5"   #no2, o3, pm10
#years           <- c(2020, 2021, 2022)
years           <- 2019
save_result     <-"/home/abarrant/esarchive/scratch/abarrant/results_regional_"   
file_format     <-"gene"   #ghost
time_resolution <-"daily"  #"hourly"
compute_in_UTM  <- "TRUE"
crs_latlon      <- "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0"
crs_utm         <- "+proj=utm +zone=31 +ellps=intl +units=m +no_defs"

path_gbm    <- "/home/abarrant/esarchive/scratch/abarrant/results_regional_2019/GBM_eval/"  ##change
############################################################
#                                                          #
#                 Load Local Stations                      #
#                                                          #
############################################################
#We will pick the latest data available
local_stations     <- readRDS(paste0(save_result,years[length(years)],"/results_gene_",pollutant, "/output/","local_stations_",file_format, "_",pollutant,".rds"))

############################################################
#                                                          #
#                  Load All observations                   #
#                                                          #
############################################################

df_obs_years <- NULL
for(read_year in years){
  df_obs_all        <- readRDS(file = paste0(save_result,read_year,"/results_gene_",pollutant, "/output/","df_obs_",file_format,"_",pollutant,".rds"))
  
  start_date  <- as.Date(paste(read_year, "-01-01", sep = ""))
  end_date    <- as.Date(paste(read_year, "-12-31", sep = ""))
  
  dates      <- seq(start_date, end_date, by = "day")                   
  df_obs_sub <- subset(df_obs_all, date>=start_date & date <= end_date) 
  
  
  # Comprovation we don't have duplications in obs
  if(any(duplicated(df_obs_sub$date))){
    print(paste0("Warning, there is duplicated observations on date :", df_obs_sub$date[duplicated(df_obs_sub$date)]))
    df_obs_sub <- distinct(df_obs_sub, date, .keep_all = TRUE)
    print("The df_obs_sub (alias=observations) have been modified to have the same length as the model")
  }
  
  if (length(df_obs_sub$date) != length(dates)){
    print("WARNING: There are missing hours/dates in observations. And the code is not developed for daily data yet!!")
  }
  
  df_obs_years <- rbind.fill(df_obs_years, df_obs_sub) #combine df by row, filling in missing columns
}

names_local_stations <- as.list(intersect(local_stations@data$ref_obs, colnames(df_obs_years)))

############################################################
#                                                          #
#            Load raw model interpolated to AQMS           #
#                                                          #
############################################################
df_all_model_raw_at_AQMS <- NULL
for(read_year in years){
  
  model_raw_at_AQMS_df     <- readRDS(paste0(save_result,read_year,"/results_gene_",pollutant, "/output/","model_raw_at_AQMS_",file_format, "_",pollutant,"_df.rds"))
  df_all_model_raw_at_AQMS <- rbind.fill(df_all_model_raw_at_AQMS, model_raw_at_AQMS_df)
}

############################################################
#                                                          #
#   AQMS interpolated to AQMS as covariate for GBM         #
#                                                          #
############################################################

# We pick up the observations configured in config file:
obs_filled       <- df_obs_years 
AQMS_obs         <- subset(df_obs_years, select = -date) #we remove the column dates from observations data frame


#per una estacio concreta  -------
station <- "ES1754A"                           #la senia (ES1754A, 165 obs) // Bcn Eixample (ES1438A, 317 obs)
# Select station data (PREDICTORS)
single_station_data <- data.frame(date            = df_all_model_raw_at_AQMS$date,
                                  obs             = AQMS_obs[,station],                     #valor obs a l'estacio (de vegades nan)
                                  model           = df_all_model_raw_at_AQMS[,station])              #predictor del valor interpolat a lestacio a partir de totes les altres 


single_station_data                     <- na.omit(single_station_data)  #ometre gaps nan



#BENCHMARK ------------




#generem dos dataframes buits per posar els valors estadistics de cada iteracio
#Stats_persist <- data.frame(gap_size = numeric(1), train_fraction = numeric(1), MB = numeric(1), r = numeric(1), RMSE = numeric(1), seed = numeric(1))
#Stats_linterpol <- data.frame(gap_size = numeric(1), train_fraction = numeric(1), MB = numeric(1), r = numeric(1), RMSE = numeric(1), seed = numeric(1))

Stats_persist <- NULL
Stats_linterpol <- NULL
array_ind <- NULL

#rang de gaps
gap_sizes <- 1:10  #fer fins a mida gap 20-30
#gap_start_values <- c(0.6, rep(0.5, 5))  #limit inferior de la train_fraction en funcio de les bosses i els valors disponibles


start_time <- Sys.time()

for (i_seed in 1:100){
  #i_seed <- 5
  set.seed(i_seed)
  print(i_seed)
  
  #bucle per cada gap
  for (i in seq_along(gap_sizes)) {
    
    gap_size <- gap_sizes[i]             #agafar mida gap
    #gap_start <- gap_start_values[i]     #agafar limit inferior train fraction 
    
    #gap_size <- 7
    #gap_start <- 0.7
    
    #bucle de cada train fraction 
    for (train_fraction in seq(0.1, 0.9, by = 0.1)) {
      
      #train_fraction <- 0.3
      #sample size, numero de files segons la train fraction
      smp_size <- floor(train_fraction * nrow(single_station_data))
      
      #generem dataframes per posar les dades
      data_training <- data.frame(date = single_station_data[, 'date'], Obs_Train = single_station_data[, 'obs'])
      data_test <- data.frame(date = single_station_data[, 'date'], Obs_Train = single_station_data[, 'obs'])
      
      #generem dataframes per posar les dades de les observacions i els gaps
      Obs_Train <- data.frame(date = single_station_data[, 'date'], Obs_Train = single_station_data[, 'obs'])
      Obs_Test <- data.frame(date = single_station_data[, 'date'], Obs_Test = NA)
      
      array_ind <- NULL
      
      #bucle per generar els gaps (fins que el nombre de files a data_training es menor a la smp_size)
      while (nrow(data_training) >= smp_size && length(unique(array_ind)) < nrow(Obs_Train)  ) {
        
        ind <- sample(seq_len(nrow(Obs_Train)), size = 1)  #agafem un index random de mida 1, del df data_training_bags, on hem exclos els valors del contron del gap
        array_ind <- c(array_ind, ind)
        #ind_date <- sample(Obs_Train$date, size = 1)
        #ind <- which(Obs_Train$date==ind_date)
        
        #condicions de bon index escollit per crear gap (ni al final, i mirant valors anteriors i posteriors interiors al gap)
        if (ind > 1 && ind <= (nrow(Obs_Train) - gap_size) && !is.na(Obs_Train$Obs_Train[ind - 1]) && !is.na(Obs_Train$Obs_Train[ind]) && !is.na(Obs_Train$Obs_Train[ind + gap_size])) {
          data_test <- Obs_Train[ind:(ind + gap_size - 1), ] %>% arrange(date)                 #agafem les dades del gap, en funcio mida gap
          Obs_Test[Obs_Test$date %in% data_test$date, "Obs_Test"] <- data_test[, "Obs_Train"]  #col.loquem les dades dels gaps
          Obs_Train[Obs_Train$date %in% data_test$date, "Obs_Train"] <- NA                     #fem el forat
          data_training <- na.omit(Obs_Train)                                                  #df amb nomes els valors sense gaps
        }
      }
      
      #si ha arribat al limit de train_fraction, provar una altra
      if (length(unique(array_ind)) == nrow(Obs_Train) ){
        next
      }
      
      #generem un df comu amb els valors de training, test(gaps) i els metodes de gap filling
      Obs_Train_station <- data.frame(date = single_station_data[, 'date'], Obs_Train = NA)
      Obs_Train_station[Obs_Train_station$date %in% Obs_Train$date, "Obs_Train"] <- Obs_Train[, "Obs_Train"]
      Obs_Train_station[Obs_Train_station$date %in% Obs_Test$date, "Obs_Test"] <- Obs_Test[, "Obs_Test"]
      
      #columnes persistance i linear interpol
      Obs_Persist <- data.frame(date = Obs_Train_station[, 'date'], Obs_Persist = NA)
      Obs_Linterpol <- data.frame(date = Obs_Train_station[, 'date'], Obs_Linterpol = NA)
      Obs_Train_station[, "Obs_Persist"] <- Obs_Persist[, "Obs_Persist"]
      Obs_Train_station[, "Obs_Linterpol"] <- Obs_Linterpol[, "Obs_Linterpol"]
      
      
      #col.loquem lultim valor disponible de la data training a la columna de persistance
      Obs_Train_station$Obs_Persist <- na.locf(Obs_Train_station$Obs_Train)
      
      #fem una interpolacio lineal entre els valors extrems de la data training i els afegim a linear interpol (q passa si el primer es nan)
      Obs_Train_station$Obs_Linterpol <- approx(x = which(!is.na(Obs_Train_station$Obs_Train)),
                                                y = Obs_Train_station$Obs_Train[!is.na(Obs_Train_station$Obs_Train)],
                                                xout = 1:length(Obs_Train_station$date))$y
      
      #dades dels gaps
      data_test <- na.omit(Obs_Test)
      
      #generem un df amb nomes les dades dels gaps i els resultats del gap-filling
      Results_Test <- data.frame(date = data_test[, 'date'], Obs_Test = data_test$Obs_Test)
      Results_Test$Obs_Persist <- Obs_Train_station[Obs_Train_station$date %in% Results_Test$date, "Obs_Persist"]
      Results_Test$Obs_Linterpol <- Obs_Train_station[Obs_Train_station$date %in% Results_Test$date, "Obs_Linterpol"]
      
      #calcul parametres estadistics
      Summary_stats_persist <- openair::modStats(Results_Test, obs = "Obs_Test", mod = "Obs_Persist")
      Summary_stats_linterpol <- openair::modStats(Results_Test, obs = "Obs_Test", mod = "Obs_Linterpol")
      
      #afegim (append) els parametres stats al df comu 
      Stats_persist <- rbind(Stats_persist, c(gap_size = gap_size, train_fraction = train_fraction, MB = Summary_stats_persist$MB, r = Summary_stats_persist$r, RMSE = Summary_stats_persist$RMSE, seed = i_seed))
      Stats_linterpol <- rbind(Stats_linterpol, c(gap_size = gap_size, train_fraction = train_fraction, MB = Summary_stats_linterpol$MB, r= Summary_stats_linterpol$r, RMSE = Summary_stats_linterpol$RMSE, seed = i_seed))
      
      #plot dels resultats als gaps 
      #matplot(Results_Test$date, Results_Test[, -1], type = "h", pch=20,  col = c("red", "green", "blue"), xlab = "Date", ylab = "PM2.5 (\u03BCm/m³)", main = paste0("Test Results gap-filling (",gap_size, ",", train_fraction, ")"))
      #legend("topright", legend = colnames(Results_Test)[-1], col = c("red", "green", "blue"), lty = 1, cex = 0.8)
      
      #matplot(Results_Test$Obs_Test, Results_Test$Obs_Linterpol, type = "p", pch=20, xlab = "obs_test", ylab = "obs_linterpol", main = paste0("Test Results gap-filling (",gap_size, ",", train_fraction, ")"))
      #matplot(Results_Test$Obs_Test, Results_Test$Obs_Persist, type = "p", pch=20, xlab = "obs_test", ylab = "obs_persist", main = paste0("Test Results gap-filling (",gap_size, ",", train_fraction, ")"))
      #matplot(Results_Test$Obs_Persist, Results_Test$Obs_Linterpol, type = "p", pch=20, xlab = "obs_persist", ylab = "obs_linterpol", main = paste0("Test Results gap-filling (",gap_size, ",", train_fraction, ")"))
      
    }
  }
}

end_time <- Sys.time()
time <- end_time - start_time
print(time)



mean_linter <- Stats_linterpol %>%
  as.data.frame() %>%
  group_by(gap_size, train_fraction) %>%
  dplyr::summarise(r_mean= mean(r), r_std = sd(r), MB_mean= mean(MB), MB_std = sd(MB), RMSE_mean = mean(RMSE), RMSE_std = sd(RMSE))

mean_persist <- Stats_persist %>%
  as.data.frame() %>%
  group_by(gap_size, train_fraction) %>%
  dplyr::summarise(r_mean= mean(r), r_std = sd(r), MB_mean= mean(MB), MB_std = sd(MB), RMSE_mean = mean(RMSE), RMSE_std = sd(RMSE))

# r mean value -----------------------------------------------------------------


ggplot(mean_linter, aes(x = gap_size, y = r_mean, color = as.factor(train_fraction))) +
  geom_line() +
  labs(title = "Stats linear interpol",
       x = "gap_size",
       y = "r_mean", 
       color = "Train fraction") +
  scale_color_viridis_d() +  # paleta de colores magma?
  theme_minimal() +
  ylim(0, 1) 

ggplot(mean_persist, aes(x = gap_size, y = r_mean, color = as.factor(train_fraction))) +
  geom_line() +
  labs(title = "Stats persistance",
       x = "gap_size",
       y = "r_mean", 
       color = "Train fraction") +
  scale_color_viridis_d() + 
  theme_minimal() +
  ylim(0, 1) 

#graph 2d en funcio de la gap size
#plot_ly(mean_linter, x = ~gap_size, y = ~r_mean) %>%
 # add_markers(color = ~train_fraction, colors = c("orange", "red", "purple", "blue")) %>%
 # layout(title = "Stats linear interpol", xaxis = list(title = "gap_size"), yaxis = list(title = "r", range = c(0,1)))


# MB mean value ----------------------------------------------------------------
ggplot(mean_linter, aes(x = gap_size, y = MB_mean, color = as.factor(train_fraction))) +
  geom_line() +
  labs(title = "Stats linear interpol",
       x = "gap_size",
       y = "MB_mean", 
       color = "Train fraction") +
  scale_color_viridis_d() +
  theme_minimal()  +
  ylim(-0.4, 0.4) #ajustar limits 

ggplot(mean_persist, aes(x = gap_size, y = MB_mean, color = as.factor(train_fraction))) +
  geom_line() +
  labs(title = "Stats persistance",
       x = "gap_size",
       y = "MB_mean", 
       color = "Train fraction") +
  scale_color_viridis_d() +  
  theme_minimal() +
  ylim(-0.4, 0.4) 


# RMSE mean value ---------------------------------------------------------------

ggplot(mean_linter, aes(x = gap_size, y = RMSE_mean, color = as.factor(train_fraction))) +
  geom_line() +
  labs(title = "Stats linear interpol",
       x = "gap_size",
       y = "RMSE_mean", 
       color = "Train fraction") +
  scale_color_viridis_d() +
  theme_minimal()  #+
  #ylim(-0.4, 0.4) #ajustar limits 

ggplot(mean_persist, aes(x = gap_size, y = RMSE_mean, color = as.factor(train_fraction))) +
  geom_line() +
  labs(title = "Stats persistance",
       x = "gap_size",
       y = "RMSE_mean", 
       color = "Train fraction") +
  scale_color_viridis_d() +  
  theme_minimal() #+
  #ylim(-0.4, 0.4) 





#saving
saveRDS(object=mean_persist, file=paste0(path_gbm,"mean_persist.rds"))
saveRDS(object=mean_linter, file=paste0(path_gbm,"mean_linterpol.rds"))
#benchmark_results <- readRDS(file=paste0(path_gbm,"benchmark_results.rds"))



#generar grafic 3d
plot_ly(Stats_linterpol, x = ~gap_size, y = ~train_fraction, z = ~r) %>%
  add_markers(color = ~gap_size) %>%
  layout(title = "Stats linear interpol")

plot_ly( Stats_persist, x = ~gap_size, y = ~train_fraction, z = ~r)%>%
  add_markers(color = ~gap_size) %>%
  layout(title = "Stats persistance")


